/*
 * Copyright notice from original mutt:
 * Copyright (C) ????-2004 Justin Hibbits <jrh29@po.cwru.edu>
 * Copyright (C) 2004 Thomer M. Gil <mutt@thomer.com>
 *
 * Parts were written/modified by:
 * Rocco Rutte <pdmef@cs.tu-berlin.de>
 * Nico Golde <nico@ngolde.de>
 *
 * This file is part of mutt-ng, see http://www.muttng.org/.
 * It's licensed under the GNU General Public License,
 * please see the file GPL in the top level source directory.
 */

#include <lib-ui/lib-ui.h>
#include <libgen.h>

#include <lib-ui/menu.h>
#include <lib-ui/sidebar.h>

#include "mutt.h"
#include "charset.h"
#include "buffy.h"
#include "keymap.h"

static int CurBuffy = 0;

/* computes first entry to be shown */
static int calc_boundaries(void)
{
    int TopBuffy = 0;

    if (CurBuffy < 0 || CurBuffy >= Incoming.len)
        CurBuffy = 0;

    if (option(OPTSIDEBARNEWMAILONLY)) {
        int n = 0;

        for (int i = 0; i < CurBuffy; i++) {
            n += Incoming.arr[i]->new > 0;
        }

        n %= (LINES - 3);
        if (!n)
            return CurBuffy;

        for (int i = CurBuffy - 1; i >= 0; i--) {
            if (Incoming.arr[i]->new > 0 && --n <= 0) {
                return i;
            }
        }
    } else {
        TopBuffy = CurBuffy - (CurBuffy % (LINES - 3));
    }

    return TopBuffy < 0 ? 0 : TopBuffy;
}

static char *shortened_hierarchy (char *hbox, int maxlen)
{
  int dots = 0;
  char *last_dot = NULL;
  int i, j, len = m_strlen(hbox);
  char *new_box;

  if (!SidebarBoundary || !*SidebarBoundary)
    return (m_strdup(hbox));

  for (i = 0; i < len; ++i) {
    if (strchr (SidebarBoundary, hbox[i])) {
      ++dots;
      last_dot = &hbox[i];
    }
  }

  if (last_dot) {
    ++last_dot;
    new_box = p_new(char, maxlen + 1);
    new_box[0] = hbox[0];
    for (i = 1, j = 1; j < maxlen && i < len; ++i) {
      if (strchr (SidebarBoundary, hbox[i])) {
        new_box[j++] = hbox[i];
        new_box[j] = 0;
        if (&hbox[i + 1] != last_dot || j + m_strlen(last_dot) > maxlen) {
          new_box[j++] = hbox[i + 1];
          new_box[j] = 0;
        } else {
          m_strcat(&new_box[j], maxlen + 1, last_dot);
          break;
        }
      }
    }
    return new_box;
  }
  return m_strdup(hbox);
}

static const char *
sidebar_number_format(char* dest, ssize_t destlen,
                      char op, const char* src, const char* fmt,
                      const char* ifstr, const char* elstr,
                      anytype data, format_flag flags)
{
  char tmp[STRING];
  BUFFY* b = Incoming.arr[data.i];
  int opt = flags & M_FORMAT_OPTIONAL;
  int c = Context && !m_strcmp(Context->path, b->path);

  switch (op) {
    /* deleted */
    case 'd':
      if (!opt) {
        snprintf (tmp, sizeof (tmp), "%%%sd", fmt);
        snprintf (dest, destlen, tmp, c ? Context->deleted : 0);
      } else if ((c && Context->deleted == 0) || !c)
        opt = 0;
      break;
    /* flagged */
    case 'F':
    case 'f':                   /* for compatibility */
      if (!opt) {
        snprintf (tmp, sizeof (tmp), "%%%sd", fmt);
        snprintf (dest, destlen, tmp, c ? Context->flagged : b->msg_flagged);
      } else if ((c && Context->flagged == 0) || (!c && b->msg_flagged == 0))
        opt = 0;
      break;
    /* total */
    case 'c':                   /* for compatibility */
    case 'm':
      if (!opt) {
        snprintf (tmp, sizeof (tmp), "%%%sd", fmt);
        snprintf (dest, destlen, tmp, c ? Context->msgcount : b->msgcount);
      } else if ((c && Context->msgcount == 0) || (!c && b->msgcount == 0))
        opt = 0;
      break;
    /* total shown, i.e. not hidden by limit */
    case 'M':
      if (!opt) {
        snprintf (tmp, sizeof (tmp), "%%%sd", fmt);
        snprintf (dest, destlen, tmp, c ? Context->vcount : 0);
      } else if ((c && Context->vcount == 0) || !c)
        opt = 0;
      break;
    /* new */
    case 'n':
      if (!opt) {
        snprintf (tmp, sizeof (tmp), "%%%sd", fmt);
        snprintf (dest, destlen, tmp, c ? Context->new : b->new);
      } else if ((c && Context->new == 0) || (!c && b->new == 0))
        opt = 0;
      break;
    /* unread */
    case 'u':
      if (!opt) {
        snprintf (tmp, sizeof (tmp), "%%%sd", fmt);
        snprintf (dest, destlen, tmp, c ? Context->unread : b->msg_unread);
      } else if ((c && Context->unread == 0) || (!c && b->msg_unread == 0))
        opt = 0;
      break;
    /* tagged */
    case 't':
      if (!opt) {
        snprintf (tmp, sizeof (tmp), "%%%sd", fmt);
        snprintf (dest, destlen, tmp, c ? Context->tagged : 0);
      } else if ((c && Context->tagged == 0) || !c)
        opt = 0;
      break;
  }

  if (flags & M_FORMAT_OPTIONAL)
    m_strformat(dest, destlen, 0, opt ? ifstr : elstr,
                sidebar_number_format, data, flags);
  return src;
}

int sidebar_need_count(void)
{
    return ui_layout_sidebar_w() && !m_strisempty(SidebarNumberFormat);
}

/* print single item
 * returns:
 *      0       item was not printed ('cause of $sidebar_newmail_only)
 *      1       item was printed
 */
static int make_sidebar_entry(WINDOW *sw, char *sbox, int idx, ssize_t len)
{
    int shortened = 0, lencnt = 0;
    char no[STRING], entry[STRING];
    int l_m = m_strlen(Maildir);

    if (option(OPTSIDEBARNEWMAILONLY) && sbox && Context && Context->path
    &&  m_strcmp(Context->path, sbox) && Incoming.arr[idx]->new == 0)
        return 0;

    m_strformat(no, sizeof(no), len, SidebarNumberFormat,
                sidebar_number_format, idx, 0);
    lencnt = m_strlen(no);

    if (l_m > 0 && m_strncmp(sbox, Maildir, l_m) == 0 && m_strlen(sbox) > l_m)
    {
        sbox += l_m;
        if (Maildir[strlen(Maildir) - 1] != '/') {
            sbox += 1;
        }
    } else {
        sbox = basename(sbox);
    }

    if (option(OPTSHORTENHIERARCHY) && m_strlen(sbox) > len - lencnt - 1) {
        sbox = shortened_hierarchy(sbox, len - lencnt - 1);
        shortened = 1;
    }

    snprintf(entry, sizeof(entry), "%*s", (int)len, no);
    memcpy(entry, sbox, MIN(len, m_strlen(sbox)));
    waddnstr(sw, entry, len);

    if (shortened)
        p_delete(&sbox);

    return 1;
}

/* returns folder name of currently 
 * selected folder for <sidebar-open>
 */
const char *sidebar_get_current(void)
{
    if (!Incoming.len)
        return NULL;
    return Incoming.arr[CurBuffy]->path;
}

/* internally sets item to buf */
void sidebar_set_current(const char* buf)
{
    int i = buffy_lookup(buf);
    if (i >= 0) {
        CurBuffy = i;
    }
}

/* fix counters for a context
 * FIXME since ctx must not be of our business, move it elsewhere
 */
void sidebar_set_buffystats (CONTEXT* curContext) {
  int i = 0;
  BUFFY* tmp = NULL;
  if (!curContext || !Incoming.len || (i = buffy_lookup (curContext->path)) < 0)
    return;
  tmp = Incoming.arr[i];
  tmp->new = curContext->new;
  tmp->msg_unread = curContext->unread;
  tmp->msgcount = curContext->msgcount;
  tmp->msg_flagged = curContext->flagged;
}

int sidebar_draw(void)
{
    static short initialized = false, prev_show_value = 0;
    int x, y, i, line;
    char blank[STRING];
    WINDOW *sw;

    /* initialize first time */
    if (!initialized) {
        prev_show_value = option(OPTMBOXPANE);
        initialized = 1;
    }

    /* save or restore the value SidebarWidth */
    if (prev_show_value != option(OPTMBOXPANE)) {
        if (!prev_show_value && option(OPTMBOXPANE)) {
            /* after toggle: force recounting of all mail */
            buffy_check (2);
        }
        prev_show_value = option(OPTMBOXPANE);
    }

    sw = ui_layout_sidebar_w();
    if (!sw)
        return 0;
    getmaxyx(sw, y, x);

    memset(&blank, ' ', sizeof(blank));

    wmove(sw, 0, 0);
    WSETCOLOR(sw, MT_COLOR_STATUS);
    waddnstr(sw, blank, x);

    line = 1;
    for (i = calc_boundaries(); i < Incoming.len && line < y - 1; i++) {
        BUFFY *tmp = Incoming.arr[i];

        if (i == CurBuffy)
            WSETCOLOR(sw, MT_COLOR_INDICATOR);
        else if (tmp->new > 0)
            WSETCOLOR(sw, MT_COLOR_NEW);
        else if (tmp->msg_flagged > 0)
            WSETCOLOR(sw, MT_COLOR_FLAGGED);
        else
            WSETCOLOR(sw, MT_COLOR_NORMAL);

        if (make_sidebar_entry(sw, tmp->path, i, x - 1)) {
            WSETCOLOR(sw, MT_COLOR_SIDEBAR);
            waddch(sw, ACS_VLINE);
            line++;
        }
    }

    while (line < y - 1) {
        WSETCOLOR(sw, MT_COLOR_NORMAL);
        waddnstr(sw, blank, x - 1);
        WSETCOLOR(sw, MT_COLOR_SIDEBAR);
        waddch(sw, ACS_VLINE);
        line++;
    }

    WSETCOLOR(sw, MT_COLOR_STATUS);
    waddnstr(sw, blank, x);
    WSETCOLOR(sw, MT_COLOR_NORMAL);

    return 0;
}

void sidebar_scroll(int op)
{
    if (!Incoming.len)
        return;

    switch (op) {
      case OP_SIDEBAR_NEXT:
        if (!option(OPTSIDEBARNEWMAILONLY)) {
            CurBuffy = (CurBuffy + 1) % Incoming.len;
            break;
        }
        /* FALLTHROUGH */
      case OP_SIDEBAR_NEXT_NEW:
        for (int i = 1; i < Incoming.len; i++) {
            if (Incoming.arr[(CurBuffy + i) % Incoming.len]->new > 0) {
                CurBuffy = (CurBuffy + i) % Incoming.len;
                break;
            }
        }
        break;

      case OP_SIDEBAR_PREV:
        if (!option(OPTSIDEBARNEWMAILONLY)) {
            if (!CurBuffy) {
                CurBuffy = Incoming.len;
            }
            CurBuffy--;
            break;
        }
        /* FALLTHROUGH */
      case OP_SIDEBAR_PREV_NEW:
        for (int i = Incoming.len - 1; i > 0; i--) {
            if (Incoming.arr[(CurBuffy + i) % Incoming.len]->new > 0) {
                CurBuffy = (CurBuffy + i) % Incoming.len;
                break;
            }
        }
        break;

      case OP_SIDEBAR_SCROLL_UP:
        CurBuffy -= LINES - 3;
        if (CurBuffy < 0)
            CurBuffy = 0;
        break;
      case OP_SIDEBAR_SCROLL_DOWN:
        CurBuffy += LINES - 3;
        if (CurBuffy >= Incoming.len)
            CurBuffy = Incoming.len - 1;
        break;
      default:
        return;
    }

    sidebar_draw();
}
